@@ -1,13 +1,14 @@
-/*  $Revision: 1.29 $
+/*  $Revision: 1.38 $
 **
 **  Send an article (prepared by someone on the local site) to the
 **  master news server.
 */
-#include "configdata.h"
 #include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include <ctype.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <pwd.h>
 #include <grp.h>
@@ -19,7 +20,6 @@
 #include "nntp.h"
 #include "paths.h"
 #include "libinn.h"
-#include "clibrary.h"
 #include "macros.h"
 
 
@@ -45,6 +45,8 @@ typedef struct _HEADER {
 } HEADER;
 
 
+STATIC BOOL	Dump;
+STATIC BOOL	Revoked;
 STATIC BOOL	Spooling;
 STATIC char	SPOOLNEWS[] = _PATH_SPOOLNEWS;
 STATIC char	**OtherHeaders;
@@ -95,6 +97,8 @@ STATIC HEADER	Table[] = {
 #define _references	15
     {	"Reply-To",		TRUE,	HTstd },
 #define _replyto	16
+    {	"Also-Control",		TRUE,	HTstd },
+#define _alsocontrol	17
     {	"Xref",			FALSE,	HTstd },
     {	"Summary",		TRUE,	HTstd },
     {	"Keywords",		TRUE,	HTstd },
@@ -121,6 +125,8 @@ QuitServer(x)
 
     if (Spooling)
 	exit(x);
+    if (x)
+	(void)fprintf(stderr, "(Article not posted.)\n");
     (void)fprintf(ToServer, "quit\r\n");
     if (FLUSH_ERROR(ToServer)) {
 	(void)fprintf(stderr, "Can't send quit to server, %s",
@@ -376,6 +382,9 @@ AnAdministrator(name, group)
     char		**mem;
     char		*p;
 
+    if (Revoked)
+	return FALSE;
+
     /* Find out who we are. */
     if ((pwp = getpwnam(NEWSUSER)) == NULL)
 	/* Silent falure; clients might not have the group. */
@@ -421,10 +430,6 @@ CheckControl(ctrl, pwp)
     save = *p;
     *p = '\0';
 
-    if (EQ(ctrl, "ihave") || EQ(ctrl, "sendme")) {
-	(void)fprintf(stderr, "Unsupported \"%s\" control message.\n", ctrl);
-	QuitServer(1);
-    }
     if (EQ(ctrl, "cancel")) {
 	for (q = p + 1; ISWHITE(*q); q++)
 	    continue;
@@ -436,13 +441,15 @@ CheckControl(ctrl, pwp)
 	    CheckCancel(q, FALSE);
     }
     else if (EQ(ctrl, "checkgroups")
+	  || EQ(ctrl, "ihave")
+	  || EQ(ctrl, "sendme")
 	  || EQ(ctrl, "newgroup")
 	  || EQ(ctrl, "rmgroup")
 	  || EQ(ctrl, "sendsys")
 	  || EQ(ctrl, "senduuname")
 	  || EQ(ctrl, "version")) {
 	(void)strcpy(name, pwp->pw_name);
-	if (!AnAdministrator(name, pwp->pw_gid)) {
+	if (!AnAdministrator(name, (int)pwp->pw_gid)) {
 	    (void)fprintf(stderr,
 		    "Ask your news administrator to do the \"%s\" for you.\n",
 		    ctrl);
@@ -478,6 +485,19 @@ FormatUserName(pwp, node)
     char	*out;
     char	*p;
 
+#if	!defined(DONT_MUNGE_GETENV)
+    (void)memset(outbuff, 0, SMBUF);
+    if ((p = getenv("NAME")) != NULL) {
+	(void)strncpy(outbuff, p, SMBUF);
+	outbuff[SMBUF - 1] = '\0';	/* paranoia */
+    }
+    if (strlen(outbuff) == 0) {
+#endif	/* !defined(DONT_MUNGE_GETENV) */
+
+
+#if	defined(DONT_MUNGE_GECOS)
+    (void)strcpy(outbuff, pwp->pw_gecos);
+#else
     p = pwp->pw_gecos;
     if (*p == '*')
 	p++;
@@ -498,10 +518,15 @@ FormatUserName(pwp, node)
 	    *out++ = *p;
     }
     *out = '\0';
-    out = TrimSpaces(outbuff);
+#endif	/* defined(DONT_MUNGE_GECOS) */
 
-    if (outbuff[0])
-	(void)sprintf(buff, "%s@%s (%s)", pwp->pw_name, node, outbuff);
+#if	!defined(DONT_MUNGE_GETENV)
+    }
+#endif	/* !defined(DONT_MUNGE_GETENV) */
+
+    out = TrimSpaces(outbuff);
+    if (out[0])
+	(void)sprintf(buff, "%s@%s (%s)", pwp->pw_name, node, out);
     else
 	(void)sprintf(buff, "%s@%s", pwp->pw_name, node);
     return COPY(buff);
@@ -536,7 +561,8 @@ CheckDistribution(p)
 **  Process all the headers.  FYI, they're done in RFC-order.
 */
 STATIC void
-ProcessHeaders(linecount, pwp)
+ProcessHeaders(AddOrg, linecount, pwp)
+    BOOL		AddOrg;
     int			linecount;
     struct passwd	*pwp;
 {
@@ -572,11 +598,11 @@ ProcessHeaders(linecount, pwp)
     if (HDR(_from) == NULL)
 	HDR(_from) = FormatUserName(pwp, p);
     else {
-	(void)sprintf(buff, "%s@%s", pwp->pw_name, p);
-	(void)strcpy(from, HDR(_from));
-	HeaderCleanFrom(from);
-	if (!EQ(from, buff))
-	    HDR(_sender) = COPY(buff);
+      (void)sprintf(buff, "%s@%s", pwp->pw_name, p);
+      (void)strcpy(from, HDR(_from));
+      HeaderCleanFrom(from);
+      if (!EQ(from, buff))
+        HDR(_sender) = COPY(buff);
     }
 
     /* Set Date. */
@@ -598,7 +624,7 @@ ProcessHeaders(linecount, pwp)
 	p = &SIGNS[1];
 	zone = Now.tzone;
     }
-    (void)sprintf(buff, "%d %3.3s %d %02.2d:%02.2d:%02.2d %c%04.4d",
+    (void)sprintf(buff, "%d %3.3s %d %02d:%02d:%02d %c%04d",
 	tm->tm_mday, &MONTHS[3 * tm->tm_mon], 1900 + tm->tm_year,
 	tm->tm_hour, tm->tm_min, tm->tm_sec,
 	*p, (int)((zone / 60) * 100 + (zone % 60)));
@@ -609,8 +635,6 @@ ProcessHeaders(linecount, pwp)
     /* Set Subject; Control overrides the subject. */
     if (HDR(_control)) {
 	CheckControl(HDR(_control), pwp);
-	HDR(_subject) = NEW(char, 5 + strlen(HDR(_control)) + 1);
-	(void)sprintf(HDR(_subject), "cmsg %s", HDR(_control));
     }
     else {
 	p = HDR(_subject);
@@ -619,10 +643,8 @@ ProcessHeaders(linecount, pwp)
 		    "Required \"Subject\" header is missing or empty.\n");
 	    QuitServer(1);
 	}
-	if (EQn(p, "cmsg ", 5)) {
-	    HDR(_control) = p + 5;
-	    CheckControl(HDR(_control), pwp);
-	}
+	else if (HDR(_alsocontrol))
+	    CheckControl(HDR(_alsocontrol), pwp);
 #if	0
 	if (EQn(p, "Re: ", 4) && HDR(_references) == NULL) {
 	    (void)fprintf(stderr,
@@ -700,8 +722,9 @@ ProcessHeaders(linecount, pwp)
 	DISPOSE(p);
     }
 
-    /* Set Organization */
-    if (HDR(_organization) == NULL
+    /* Set Organization. */
+    if (AddOrg
+     && HDR(_organization) == NULL
      && (p = GetConfigValue(_CONF_ORGANIZATION)) != NULL) {
 	HDR(_organization) = COPY(p);
     }
@@ -822,6 +845,12 @@ CheckIncludedText(p, lines)
 	case '>':
 	    i++;
 	    break;
+	case '|':
+	    i++;
+	    break;
+	case ':':
+	    i++;
+	    break;
 	case '<':
 	    i--;
 	    break;
@@ -829,7 +858,7 @@ CheckIncludedText(p, lines)
 	if ((p = strchr(p + 1, '\n')) == NULL)
 	    break;
     }
-    if (i * 2 > lines) {
+    if ((i * 2 > lines) && (lines > 40)) {
 	(void)fprintf(stderr,
 		"Article not posted -- more included text than new text\n");
 	QuitServer(1);
@@ -854,7 +883,7 @@ MailArticle(group, article)
     char		buff[SMBUF];
 
     /* Try to get the address first. */
-    if ((address = GetModeratorAddress(group)) == NULL) {
+    if ((address = GetModeratorAddress(FromServer, ToServer, group)) == NULL) {
 	(void)fprintf(stderr,
 		"The \"%s\" newsgroup is moderated, but has no address;\n",
 		group);
@@ -916,6 +945,7 @@ ValidNewsgroups(hdr, F, article)
     register int	i;
     BOOL		approved;
     BOOL		mailed;
+    BOOL		FoundOne;
     char		*group;
     char		*q;
     char		buff[SMBUF];
@@ -933,6 +963,7 @@ ValidNewsgroups(hdr, F, article)
 
     /* Don't mail article if just checking Followup-To line. */
     approved = HDR(_approved) != NULL || article == NULL;
+    FoundOne = FALSE;
 
     h = DDstart(FromServer, ToServer);
     do {
@@ -945,13 +976,9 @@ ValidNewsgroups(hdr, F, article)
 	while (fgets(buff, sizeof buff, F) != NULL)
 	    if (buff[0] == *p && EQn(buff, p, i) && buff[i] == ' ')
 		break;
-	if (feof(F)) {
-	    if (IsNewgroup)
-		continue;
-	    (void)fprintf(stderr, "No such newsgroup as \"%s\".\n", p);
-	    DISPOSE(DDend(h));
-	    return FALSE;
-	}
+	if (feof(F))
+	    continue;
+	FoundOne = TRUE;
 	DDcheck(h, p);
 
 	/* Skip past the newsgroup name, the high and low counts, to find
@@ -963,7 +990,11 @@ ValidNewsgroups(hdr, F, article)
 	case NF_FLAG_OK:
 	    break;
 	case NF_FLAG_MODERATED:
-	    if (!approved) {
+	    if (Dump)
+		(void)fprintf(stderr,
+			"%s is moderated -- article would be mailed\n",
+			group);
+	    else if (!approved) {
 		mailed = MailArticle(group, article);
 		DISPOSE(DDend(h));
 		QuitServer(mailed ? 0 : 1);
@@ -992,6 +1023,12 @@ ValidNewsgroups(hdr, F, article)
 
     } while ((p = strtok((char *)NULL, NGSEPS)) != NULL);
 
+    if (!FoundOne && !IsNewgroup) {
+	(void)fprintf(stderr, "No such newsgroups as \"%s\".\n", hdr);
+	DISPOSE(DDend(h));
+	return FALSE;
+    }
+
     /* Set default distribution. */
     p = DDend(h);
     if (HDR(_distribution) == NULL && *p)
@@ -1018,7 +1055,7 @@ ReadStdin()
     size = BUFSIZ;
     article = NEW(char, size);
     end = &article[size - 3];
-    for (p = article; (i = getchar()) != EOF; *p++ = i)
+    for (p = article; (i = getchar()) != EOF; *p++ = (char)i)
 	if (p == end) {
 	    RENEW(article, char, size + BUFSIZ);
 	    p = &article[size - 3];
@@ -1053,31 +1090,14 @@ OfferArticle(buff, Authorized)
 }
 
 
-/*
-**  Make a temporary filename that is unlikely to collide with mktemp
-**  or cause problems for sites with 14-character filename limits.
-*/
-STATIC void
-TempName(dir, buff)
-    char	*dir;
-    char	*buff;
-{
-    time_t	now;
-
-    (void)time(&now);
-    now &= 0xFFFFFFFF;
-    (void)sprintf(buff, "%s/%08.8lxXXXXXX", dir, (long)now);
-    (void)mktemp(buff);
-}
-
-
 /*
 **  Spool article to temp file.
 */
 STATIC void
-Spoolit(article, Length)
+Spoolit(article, Length, deadfile)
     char		*article;
     SIZE_T		Length;
+    char		*deadfile;
 {
     static char		SPOOLTEMP[] = _PATH_SPOOLTEMP;
     register HEADER	*hp;
@@ -1085,12 +1105,18 @@ Spoolit(article, Length)
     register int	i;
     char		temp[BUFSIZ];
     char		buff[BUFSIZ];
+    struct stat		Sb;
 
-    TempName(SPOOLTEMP, temp);
-    (void)umask(0);
-    if ((i = open(temp, O_WRONLY | O_CREAT, BATCHFILE_MODE)) < 0
-     || (F = fdopen(i, "w")) == NULL)
-	PerrorExit(FALSE, "Can't create spool file");
+    /* Try to write to the spool dir, else the deadfile. */
+    if ((stat(SPOOLNEWS, &Sb) >= 0 && S_ISDIR(Sb.st_mode))
+     || (F = xfopena(deadfile)) == NULL) {
+	TempName(SPOOLTEMP, temp);
+	(void)umask(0);
+	if ((i = open(temp, O_WRONLY | O_CREAT, BATCHFILE_MODE)) < 0
+	 || (F = fdopen(i, "w")) == NULL)
+	    PerrorExit(FALSE, "Can't create spool file");
+	deadfile = NULL;
+    }
 
     /* Write the headers and a blank line. */
     for (hp = Table; hp < ENDOF(Table); hp++)
@@ -1109,10 +1135,12 @@ Spoolit(article, Length)
     if (fclose(F) == EOF)
 	PerrorExit(FALSE, "Can't close spool file");
 
-    /* Put the file in a good place. */
-    TempName(SPOOLNEWS, buff);
-    if (rename(temp, buff) < 0)
-	PerrorExit(FALSE, "Can't rename spool file");
+    if (deadfile == NULL) {
+	/* Put the file in a good place. */
+	TempName(SPOOLNEWS, buff);
+	if (rename(temp, buff) < 0)
+	    PerrorExit(FALSE, "Can't rename spool file");
+    }
 }
 
 
@@ -1136,7 +1164,6 @@ main(ac, av)
     static char		NOCONNECT[] = "Can't connect to server";
     register int	i;
     register char	*p;
-    register char	*next;
     register HEADER	*hp;
     int			j;
     int			Mode;
@@ -1144,12 +1171,12 @@ main(ac, av)
     FILE		*F;
     struct passwd	*pwp;
     char		*article;
+    char		*deadfile;
     char		buff[NNTP_STRLEN + 2];
     char		SpoolMessage[NNTP_STRLEN + 2];
-    BOOL		Dump;
     BOOL		DoSignature;
+    BOOL		AddOrg;
     SIZE_T		Length;
-    struct stat		Sb;
 
     /* Find out who we are. */
     if ((i = geteuid()) < 0)
@@ -1161,9 +1188,11 @@ main(ac, av)
     Mode = '\0';
     Dump = FALSE;
     DoSignature = TRUE;
+    AddOrg = TRUE;
+    (void)umask(NEWSUMASK);
 
     /* Parse JCL. */
-    while ((i = getopt(ac, av, "DNAVWShx:a:c:d:e:f:n:r:t:F:o:w:")) != EOF)
+    while ((i = getopt(ac, av, "DNAVWORShx:a:c:d:e:f:n:r:t:F:o:w:")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -1177,6 +1206,12 @@ main(ac, av)
 	case 'W':
 	    /* Ignore C News options. */
 	    break;
+	case 'O':
+	    AddOrg = FALSE;
+	    break;
+	case 'R':
+	    Revoked = TRUE;
+	    break;
 	case 'S':
 	    DoSignature = FALSE;
 	    break;
@@ -1223,22 +1258,16 @@ main(ac, av)
     /* Try to open a connection to the server. */
     if (NNTPremoteopen(&FromServer, &ToServer, buff) < 0) {
 	Spooling = TRUE;
-	i = errno;
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
 	if ((p = strchr(buff, '\r')) != NULL)
 	    *p = '\0';
-	if (stat(SPOOLNEWS, &Sb) < 0 || !S_ISDIR(Sb.st_mode)) {
-	    (void)fprintf(stderr, "%s, please try later\n",
-		    buff[0] ? buff : NOCONNECT);
-	    exit(1);
-	}
-
-	/* Save the reason why we can't post. */
 	(void)strcpy(SpoolMessage, buff[0] ? buff : NOCONNECT);
+	(void)sprintf(buff, "%s/dead.article", pwp->pw_dir);
+	deadfile = COPY(buff);
     }
-    /* See if we can post. */
-    if (!Spooling) {
+    else {
+	/* See if we can post. */
 	i = atoi(buff);
 
 	/* Tell the server we're posting. */
@@ -1263,16 +1292,8 @@ main(ac, av)
 	hp->Size = strlen(hp->Name);
     if (Mode == 'h')
 	article = StripOffHeaders(article);
-    for (i = 0, p = article; p; i++, p = next + 1) {
-	if ((next = strchr(p, '\n')) == NULL)
-	    break;
-	if (next - p > NNTP_STRLEN) {
-	    (void)fprintf(stderr,
-		    "Line too long (%d characters max):\n\t%.40s...\n",
-		    NNTP_STRLEN, p);
-	    QuitServer(1);
-	}
-    }
+    for (i = 0, p = article; (p = strchr(p, '\n')) != NULL; i++, p++)
+	continue;
 #if	defined(DO_CHECK_INCLUDED_TEXT)
     CheckIncludedText(article, i);
 #endif	/* defined(DO_CHECK_INCLUDED_TEXT) */
@@ -1280,7 +1301,7 @@ main(ac, av)
 	article = AppendSignature(Mode == 'h', article, pwp->pw_dir, &SigLines);
     else
 	SigLines = 0;
-    ProcessHeaders(i + SigLines, pwp);
+    ProcessHeaders(AddOrg, i + SigLines, pwp);
     Length = strlen(article);
 #if	LOCAL_MAX_ARTSIZE > 0
     if (Length > LOCAL_MAX_ARTSIZE) {
@@ -1310,7 +1331,7 @@ main(ac, av)
 	}
 
     /* Check the newsgroups. */
-    if ((F = CAopen(FromServer, ToServer)) != NULL) {
+    if ((F = CAlistopen(FromServer, ToServer, (char *)NULL)) != NULL) {
 	if (!ValidNewsgroups(HDR(_newsgroups), F, article)) {
 	    CAclose();
 	    QuitServer(1);
@@ -1326,13 +1347,6 @@ main(ac, av)
     else if (!Spooling)
 	PerrorExit(TRUE, "Can't get list of newsgroups");
 
-    if (Spooling) {
-	(void)fprintf(stderr, "Warning %s -- Article will be spooled.\n",
-		SpoolMessage);
-	Spoolit(article, Length);
-	exit(0);
-    }
-
     if (Dump) {
 	/* Write the headers and a blank line. */
 	for (hp = Table; hp < ENDOF(Table); hp++)
@@ -1351,6 +1365,13 @@ main(ac, av)
 	QuitServer(0);
     }
 
+    if (Spooling) {
+	(void)fprintf(stderr, "Warning %s -- Article will be spooled.\n",
+		SpoolMessage);
+	Spoolit(article, Length, deadfile);
+	exit(0);
+    }
+
     /* Article is prepared, offer it to the server. */
     i = OfferArticle(buff, FALSE);
     if (i == NNTP_AUTH_NEEDED_VAL) {
