@@ -1,4 +1,4 @@
-/*  $Revision: 1.15 $
+/*  $Revision: 1.26 $
 **
 **  Here be declarations of routines and variables in the C library.
 **  You must #include <sys/types.h> and <stdio.h> before this file.
@@ -12,22 +12,33 @@
 #include <vfork.h>
 #endif	/* defined(DO_HAVE_VFORK) */
 
+#include <sys/time.h>
+
     /* Generic pointer, used by memcpy, malloc, etc. */
     /* =()<typedef @<POINTER>@ *POINTER;>()= */
-typedef char *POINTER;
+typedef void *POINTER;
+    /* Const generic pointer, used by qsort. */
+    /* =()<typedef const @<POINTER>@ *CPOINTER;>()= */
+typedef const void *CPOINTER;
     /* What is a file offset?  Will not work unless long! */
     /* =()<typedef @<OFFSET_T>@ OFFSET_T;>()= */
 typedef long OFFSET_T;
     /* What is the type of an object size? */
     /* =()<typedef @<SIZE_T>@ SIZE_T;>()= */
-typedef int SIZE_T;
+typedef size_t SIZE_T;
     /* What is the type of a passwd uid and gid, for use in chown(2)? */
     /* =()<typedef @<UID_T>@ UID_T;>()= */
-typedef int UID_T;
+typedef uid_t UID_T;
     /* =()<typedef @<GID_T>@ GID_T;>()= */
-typedef int GID_T;
+typedef gid_t GID_T;
     /* =()<typedef @<PID_T>@ PID_T;>()= */
-typedef int PID_T;
+typedef pid_t PID_T;
+    /* =()<typedef @<U_INT32_T>@ U_INT32_T;>()= */
+typedef unsigned int U_INT32_T;
+    /* =()<typedef @<INT32_T>@ INT32_T;>()= */
+typedef int INT32_T;
+
+#include <signal.h>
     /* What should a signal handler return? */
     /* =()<#define SIGHANDLER	@<SIGHANDLER>@>()= */
 #define SIGHANDLER	void
@@ -35,13 +46,14 @@ typedef int PID_T;
 #if	defined(SIG_DFL)
     /* What types of variables can be modified in a signal handler? */
     /* =()<typedef @<SIGVAR>@ SIGVAR;>()= */
-typedef int SIGVAR;
+typedef sig_atomic_t SIGVAR;
 #endif	/* defined(SIG_DFL) */
 
 /* =()<#include @<STR_HEADER>@>()= */
 #include <string.h>
 /* =()<#include @<MEM_HEADER>@>()= */
 #include <memory.h>
+#include <stdlib.h>
 
 
 /*
@@ -66,7 +78,7 @@ typedef struct _FDSET {
 #define FD_SET(n, p)	(p)->fds_bits[(n) / NFDBITS] |= (1 << ((n) % NFDBITS))
 #define FD_CLR(n, p)	(p)->fds_bits[(n) / NFDBITS] &= ~(1 << ((n) % NFDBITS))
 #define FD_ISSET(n, p)	((p)->fds_bits[(n) / NFDBITS] & (1 << ((n) % NFDBITS)))
-#define FD_ZERO(p)	bzero((char *)(p), sizeof(*(p)))
+#define FD_ZERO(p)	(void)memset((POINTER)(p), 0, sizeof *(p))
 #endif	/* defined(FD_SETSIZE) */
 
 
@@ -82,7 +94,7 @@ typedef struct _FDSET {
 **  work on some broken compilers, sigh.
 */
 /* =()<#define FREEVAL @<FREEVAL>@>()= */
-#define FREEVAL int
+#define FREEVAL void
 
 extern int		optind;
 extern char		*optarg;
@@ -90,10 +102,13 @@ extern char		*optarg;
 extern int		errno;
 #endif	/* !defined(__STDC__) */
 
+extern char		*crypt();
 extern char		*getenv();
 extern char		*inet_ntoa();
 extern char		*mktemp();
+#if	!defined(strerror)
 extern char		*strerror();
+#endif	/* !defined(strerror) */
 extern long		atol();
 extern time_t		time();
 extern unsigned long	inet_addr();
@@ -101,6 +116,15 @@ extern FREEVAL		free();
 extern POINTER		malloc();
 extern POINTER		realloc();
 
+#if	defined(ACT_MMAP)
+#define MMAP
+#endif /* defined(ACT_MMAP) */
+#if	defined(MMAP)
+/* =()<typedef @<MMAP_PTR>@ MMAP_PTR;>()= */
+typedef caddr_t MMAP_PTR;
+extern MMAP_PTR mmap();
+#endif /* defined(MMAP) */	
+
 /* Some backward systems need this. */
 extern FILE	*popen();
 
@@ -109,18 +133,22 @@ extern FILE	*popen();
 extern int	strncasecmp();
 
 /* =()<extern @<ABORTVAL>@	abort();>()= */
-extern int	abort();
+extern void	abort();
 /* =()<extern @<ALARMVAL>@	alarm();>()= */
-extern int	alarm();
+extern unsigned int	alarm();
 /* =()<extern @<EXITVAL>@	exit();>()= */
 extern void	exit();
 /* =()<extern @<GETPIDVAL>@	getpid();>()= */
-extern int	getpid();
+extern pid_t	getpid();
 /* =()<extern @<LSEEKVAL>@	lseek();>()= */
-extern long	lseek();
+extern off_t	lseek();
 /* =()<extern @<QSORTVAL>@	qsort();>()= */
-extern int	qsort();
+extern void	qsort();
 /* =()<extern @<SLEEPVAL>@	sleep();>()= */
-extern int	sleep();
+extern unsigned int	sleep();
 /* =()<extern @<_EXITVAL>@	_exit();>()= */
-extern int	_exit();
+extern void	_exit();
+
+#if defined(DO_HAVE_SETPROCTITLE)
+extern void	setproctitle();
+#endif	/* defined(DO_HAVE_SETPROCTITLE) */
