@@ -1,10 +1,6 @@
-/* $Revision: 1.10 $
-**
-**  $Log: patchlevel.h,v $
-**  Revision 1.10  1992/08/19  16:36:04  rsalz
-**  Official release.
-**
-*/
+/*
+ * $Revision: 1.21 $
+ */
 
 
 /*
@@ -16,9 +12,9 @@
 /*
 **  Try to avoid changing these.
 */
-#define RELEASE		"1"
-#define PATCHLEVEL	"0"
-#define DATE		"20-Aug-92"
+#define RELEASE "1"
+#define PATCHLEVEL "5"
+#define DATE "28-Nov-1996"
 
 
 /*
