@@ -1,11 +1,29 @@
-/*  $Revision: 1.9 $
+/*  $Revision: 1.16 $
 **
 **  The newnews command.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "nnrpd.h"
 
+#if	defined(DONT_ALLOW_NEWNEWS)
+
+FUNCTYPE
+CMDnewnews(ac, av)
+    register int	ac;
+    char		*av[];
+{
+    Reply("%d NEWNEWS command disabled by administrator\r\n",
+	  NNTP_ACCESS_VAL);
+}
+
+#else
 
 #define FILE_LIST_DELTA		10
+#define GET_DATE(p, line)	\
+	((p = strchr(line, HIS_FIELDSEP)) == NULL ? 0 : atol(++p))
 
 
 /*
@@ -25,7 +43,9 @@ DistMatches(distribs, files)
 
     /* Loop through the file list, trying to open one of them.. */
     for (save = files[0]; *files; files++) {
-	(void)sprintf(buff, "%s/%s", _PATH_SPOOL, files[0]);
+      /* this next line makes me nervous--from unoff4... */
+	(void)sprintf(buff, "%s/%s/%s", _PATH_SPOOL, files[0],
+		files[0] + strlen(files[0]) + 1);
 	for (p = &buff[STRLEN(_PATH_SPOOL)]; *p; p++)
 	    if (*p == '.')
 		*p = '/';
@@ -33,7 +53,6 @@ DistMatches(distribs, files)
 	    break;
     }
     if (*files == NULL) {
-	syslog(L_ERROR, "%s cant fopen %s %m", ClientHost, save);
 	return FALSE;
     }
 
@@ -49,7 +68,7 @@ DistMatches(distribs, files)
 	if ((q = strchr(p, ':')) == NULL)
 	    continue;
 	*q = '\0';
-	if (caseEQ(buff, "distribution")) {
+	if (caseEQ(p, "distribution")) {
 	    for (q += 2, dp = distribs; *dp; dp++)
 		if (caseEQ(q, *dp)) {
 		    QIOclose(qp);
@@ -74,10 +93,11 @@ GetFiles(p)
     static int		size;
     static char		**list;
     register int	i;
+    register char	*slash = p;
 
     if (size == 0) {
 	size = FILE_LIST_DELTA;
-	list = NEW(char*, size);
+	list = NEW(char*, size + 1);
     }
 
     for (i = 0 ; ; ) {
@@ -86,22 +106,25 @@ GetFiles(p)
 	if (*p == '\0' || *p == '\n')
 	    break;
 
-	if (i >= size) {
+	if (i >= size - 1) {
 	    size += FILE_LIST_DELTA;
 	    RENEW(list, char *, size + 1);
 	}
-	for (list[i++] = p; *p && *p != '\n' && !ISWHITE(*p); p++)
+	for (list[i] = p; *p && *p != '\n' && !ISWHITE(*p); p++) {
 	    if (*p == '/')
-		*p = '\0';
+		*(slash = p) = '\0';
+	}
+	if (*p) *p++ = '\0';
+	if (slash > list[i])  /* Only save filenames with '/' */
+	    i++;
     }
     list[i] = NULL;
     return i ? list : NULL;
 }
 
-
 /*
 **  Seek to first line in the history file where the date is after the
-**  desired one.   Returns FALSE on failure.  Slow.
+**  desired one.   Returns FALSE on failure.
 */
 STATIC BOOL
 FindLinesAfter(date, line, linesize, F)
@@ -111,14 +134,45 @@ FindLinesAfter(date, line, linesize, F)
     FILE	*F;
 {
     char	*p;
+    long	upper;
+    long	lower;
+    long	middle;
 
-    while (fgets(line, linesize, F) != NULL) {
-	if ((p = strchr(line, HIS_FIELDSEP)) == NULL
-	 || *++p == '\n' || *p == '\0')
+    /* Read first line -- is it in our range? */
+    (void)fseek(F, 0L, SEEK_SET);
+    if (fgets(line, linesize, F) == NULL)
+	return FALSE;
+    if (GET_DATE(p, line) >= date)
+	return TRUE;
+
+    /* Set search ranges and go. */
+    lower = 0;
+    (void)fseek(F, 0L, SEEK_END);
+    upper = ftell(F);
+    for ( ; ; ) {
+	/* Seek to middle line. */
+	middle = (upper + lower) / 2;
+	(void)fseek(F, middle, SEEK_SET);
+	while (getc(F) != '\n' && ++middle <= upper)
 	    continue;
-	if (atol(p) > date)
-	    return TRUE;
+
+	if (middle >= upper)
+	    break;
+
+	if (fgets(line, linesize, F) != NULL && GET_DATE(p, line) > date)
+	    upper = middle;
+	else if (lower == middle)
+	    break;
+	else
+	    lower = middle;
     }
+
+    /* Move to lower bound; we know this will always be the start of a line. */
+    (void)fseek(F, lower, SEEK_SET);
+    while (fgets(line, linesize, F) != NULL)
+	if (GET_DATE(p, line) >= date)
+	    return TRUE;
+
     return FALSE;
 }
 
@@ -188,6 +242,13 @@ CMDnewnews(ac, av)
 	AllDists = FALSE;
     }
 
+    /* Optimization in case client asks for !* (no groups) */
+    if (!strcmp(groups[0], "!*")) {
+	Reply("%s\r\n", NNTP_NEWNEWSOK);
+	Printf(".\r\n");
+	return;
+    }
+
     if ((F = fopen(HISTORY, "r")) == NULL) {
 	syslog(L_ERROR, "%s cant fopen %s %m",
 	    ClientHost, HISTORY);
@@ -198,7 +259,7 @@ CMDnewnews(ac, av)
     Reply("%s\r\n", NNTP_NEWNEWSOK);
 
     files = NULL;
-    if (FindLinesAfter(date, line, sizeof line, F)) 
+    if (FindLinesAfter(date, line, sizeof line, F))
 	do {
 	    /* Skip two tab-separated fields. */
 	    if ((p = strchr(line, HIS_FIELDSEP)) == NULL
@@ -221,3 +282,5 @@ CMDnewnews(ac, av)
     (void)fclose(F);
     Printf(".\r\n");
 }
+
+#endif	/* defined(DONT_ALLOW_NEWNEWS) */
