@@ -1,25 +1,34 @@
-/*  $Revision: 1.14 $
+/*  $Revision: 1.31 $
 **
 **  NNTP server for readers (NNRP) for InterNetNews.
 **  This server doesn't do any real load-limiting, except for what has
 **  proven empirically necesary (i.e., look at GRPscandir).
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #define MAINLINE
-#include <signal.h>
 #include "nnrpd.h"
-#include <sys/time.h>
 #include <netdb.h>
-#if	defined(hpux)
+#include <pwd.h>
+#if	defined(HPUX)
 #include <sys/pstat.h>
-#endif	/* defined(hpux) */
-#if	defined(VAR_VARARGS)
-#include <varargs.h>
-#endif	/* defined(VAR_VARARGS) */
-#if	defined(VAR_STDARGS)
-#include <stdarg.h>
-#endif	/* defined(VAR_STDARGS) */
+#endif	/* defined(HPUX) */
 
 
+/*
+** Here is some defensive code to protect the news server from hosts,
+** mostly PC's, that sometimes make a connection and then never give
+** any commands.  The connection is abandoned, but we're never told
+** about it.  The first time the connection is read, it will have a
+** timeout value of INITIAL_TIMEOUT seconds.  All subsequent reads
+** will have the standard timeout of CLIENT_TIMEOUT seconds.
+*/
+#if !defined(INITIAL_TIMEOUT)
+#define INITIAL_TIMEOUT	10
+#endif
+
 #define CMDany		-1
 
 
@@ -43,13 +52,15 @@ BOOL	PERMdefault = FALSE;
 
 STATIC double	STATstart;
 STATIC double	STATfinish;
-#if	!defined(hpux)
+STATIC char	*PushedBack;
+#if	!defined(HPUX)
 STATIC char	*TITLEstart;
 STATIC char	*TITLEend;
-#endif	/* !defined(hpux) */
+#endif	/* !defined(HPUX) */
 STATIC SIGVAR	ChangeTrace;
 
 extern FUNCTYPE	CMDauthinfo();
+extern FUNCTYPE	CMDdate();
 extern FUNCTYPE	CMDfetch();
 extern FUNCTYPE	CMDgroup();
 STATIC FUNCTYPE	CMDhelp();
@@ -61,54 +72,60 @@ extern FUNCTYPE	CMDnextlast();
 extern FUNCTYPE	CMDpost();
 extern FUNCTYPE	CMDxgtitle();
 extern FUNCTYPE	CMDxhdr();
+extern FUNCTYPE	CMDxover();
 extern FUNCTYPE	CMDxpat();
-#if	defined(DO_DO_XTHREAD)
-extern FUNCTYPE	CMDxthread();
-#endif	/* defined(DO_DO_XTHREAD) */
+extern FUNCTYPE	CMDxpath();
+extern FUNCTYPE	CMD_unimp();
 
 STATIC char	CMDfetchhelp[] = "[MessageID|Number]";
 
 STATIC CMDENT	CMDtable[] = {
-    {	"authinfo",	CMDauthinfo,	FALSE,	3,	3,
-	"user Name|pass Password" },
-    {	"article",	CMDfetch,	FALSE,	1,	2,
+    {	"authinfo",	CMDauthinfo,	FALSE,	3,	CMDany,
+	"user Name|pass Password|generic <prog> <args>" },
+    {	"article",	CMDfetch,	TRUE,	1,	2,
 	CMDfetchhelp },
-    {	"body",		CMDfetch,	FALSE,	1,	2,
+    {	"body",		CMDfetch,	TRUE,	1,	2,
 	CMDfetchhelp },
-    {	"group",	CMDgroup,	FALSE,	2,	2,
+    {	"date",		CMDdate,	FALSE,	1,	1,
+	NULL },
+    {	"group",	CMDgroup,	TRUE,	2,	2,
 	"newsgroup" },
-    {	"head",		CMDfetch,	FALSE,	1,	2,
+    {	"head",		CMDfetch,	TRUE,	1,	2,
 	CMDfetchhelp },
     {	"help",		CMDhelp,	FALSE,	1,	CMDany,
 	NULL },
-    {	"last",		CMDnextlast,	FALSE,	1,	1,
+    {	"ihave",	CMD_unimp,	TRUE,	1,	2,
 	NULL },
-    {	"list",		CMDlist,	FALSE,	1,	2,
-	"[active|newsgroups|distributions]" },
-    {	"listgroup",	CMDgroup,	FALSE,	1,	2,
+    {	"last",		CMDnextlast,	TRUE,	1,	1,
+	NULL },
+    {	"list",		CMDlist,	TRUE,	1,	3,
+	"[active|active.times|newsgroups|distributions|distrib.pats|overview.fmt|subscriptions]" },
+    {	"listgroup",	CMDgroup,	TRUE,	1,	2,
 	"newsgroup" },
     {	"mode",		CMDmode,	FALSE,	2,	2,
 	"reader" },
-    {	"newgroups",	CMDnewgroups,	FALSE,	3,	5,
+    {	"newgroups",	CMDnewgroups,	TRUE,	3,	5,
 	"yymmdd hhmmss [\"GMT\"] [<distributions>]" },
-    {	"newnews",	CMDnewnews,	FALSE,	4,	6,
+    {	"newnews",	CMDnewnews,	TRUE,	4,	6,
 	"newsgroups yymmdd hhmmss [\"GMT\"] [<distributions>]" },
-    {	"next",		CMDnextlast,	FALSE,	1,	1,
+    {	"next",		CMDnextlast,	TRUE,	1,	1,
 	NULL },
     {	"post",		CMDpost,	TRUE,	1,	1,
 	NULL },
-    {	"stat",		CMDfetch,	FALSE,	1,	2,
+    {	"slave",	CMD_unimp,	FALSE,	1,	1,
+	NULL },
+    {	"stat",		CMDfetch,	TRUE,	1,	2,
 	CMDfetchhelp },
-    {	"xgtitle",	CMDxgtitle,	FALSE,	1,	2,
+    {	"xgtitle",	CMDxgtitle,	TRUE,	1,	2,
 	"[group_pattern]" },
-    {	"xhdr",		CMDxhdr,	FALSE,	2,	3,
+    {	"xhdr",		CMDxhdr,	TRUE,	2,	3,
 	"header [range|MessageID]" },
-    {	"xpat",		CMDxpat,	FALSE,	4,	CMDany,
+    {	"xover",	CMDxover,	TRUE,	1,	2,
+	"[range]" },
+    {	"xpat",		CMDxpat,	TRUE,	4,	CMDany,
 	"header range|MessageID pat [morepat...]" },
-#if	defined(DO_DO_XTHREAD)
-    {	"xthread",	CMDxthread,	FALSE,	1,	2,
-	"[dbinit|thread]" },
-#endif	/* defined(DO_DO_XTHREAD) */
+    {	"xpath",	CMDxpath,	TRUE,	2,	2,
+	"MessageID" },
     {	NULL }
 };
 
@@ -135,8 +152,9 @@ ExitWithStats(x)
     GRPreport();
     syslog(L_NOTICE, "%s exit articles %ld groups %ld",
 	   ClientHost, ARTcount, GRPcount);
-    syslog(L_NOTICE, "%s posts received %ld rejected %ld",
-       ClientHost, POSTreceived, POSTrejected);
+    if (POSTreceived ||  POSTrejected)
+	syslog(L_NOTICE, "%s posts received %ld rejected %ld",
+	   ClientHost, POSTreceived, POSTrejected);
     syslog(L_NOTICE, "%s times user %.3f system %.3f elapsed %.3f",
 	ClientHost, usertime, systime, STATfinish - STATstart);
     exit(x);
@@ -167,13 +185,36 @@ CMDhelp(ac, av)
 
 
 /*
-**  Overwrite the original argv so that ps(1) will show what's going on.
+**  Unimplemented catch-all.
+*/
+/* ARGSUSED0 */
+FUNCTYPE
+CMD_unimp(ac, av)
+    int		ac;
+    char	*av[];
+{
+    if (caseEQ(av[0], "ihave"))
+	Reply("%d Transfer permission denied\r\n", NNTP_AUTH_NEEDED_VAL);
+    else if (caseEQ(av[0], "slave"))
+	/* Somebody sends us this?  I don't believe it! */
+	Reply("%d Unsupported\r\n", NNTP_SLAVEOK_VAL);
+    else
+	Reply("%d %s not implemented; try help\r\n",
+	    NNTP_BAD_COMMAND_VAL, av[0]);
+}
+
+
+/*
+**  Overwrite the original argv so that ps will show what's going on.
 */
 STATIC void
 TITLEset(what)
     char		*what;
 {
-#if	!defined(hpux)
+#if	defined(DO_HAVE_SETPROCTITLE)
+    setproctitle("%s %s", ClientHost, what);
+#else
+#if	!defined(HPUX)
     register char	*p;
     register int	i;
     char		buff[BUFSIZ];
@@ -193,13 +234,16 @@ TITLEset(what)
 	*p++ = ' ';
 #else
     char		buff[BUFSIZ];
-
+    union pstun un;
+    
     (void)sprintf(buff, "(nnrpd) %s %s", ClientHost, what);
-    (void)pstat(PSTAT_SETCMD, buff, 0, 0, 0);
-#endif	/* defined(hpux) */
+    (void)pstat(PSTAT_SETCMD, un, strlen(buff), 0, 0);
+#endif	/* defined(HPUX) */
+#endif	/* defined(DO_HAVE_SETPROCTITLE) */
 }
 
 
+#if	defined(DO_NNRP_GETHOSTBYADDR)
 /*
 **  Convert an IP address to a hostname.  Don't trust the reverse lookup,
 **  since anyone can fake .in-addr.arpa entries.
@@ -255,6 +299,7 @@ Address2Name(ap, hostname, i)
 	    *p = tolower(*p);
     return TRUE;
 }
+#endif	/* defined(DO_NNRP_GETHOSTBYADDR) */
 
 
 BOOL
@@ -270,6 +315,7 @@ PERMinfile(hp, ip, user, pass, accesslist)
     register char	*p;
     register BOOL	found;
     register int	i;
+    struct passwd	*pwd;
     char		buff[BIG_BUFFER];
     char		*fields[5];
 
@@ -300,18 +346,27 @@ PERMinfile(hp, ip, user, pass, accesslist)
 	    /* Malformed line. */
 	    continue;
 
-	if (hp) 
+	if (hp)
 	    /* Got an address; try to match either the IP address or as
 	     * a text hostname. */
 	    if (!(ip && wildmat(ip, fields[0])) && !wildmat(hp, fields[0]))
 		continue;
-	if (user && (!EQ(user, fields[2]) || !EQ(pass, fields[3])))
+
+	/* See if we should lookup a specific user in the passwd file */
+	if (user && pass && EQ(fields[2], "+")) {
+	    if ((pwd = getpwnam(user)) == NULL)
+		continue;
+	    if (!EQ(pwd->pw_passwd, crypt(pass, pwd->pw_passwd)))
+		continue;
+	}
+	/* Matching for a specific user or just the host? */
+	else if (user && (!EQ(user, fields[2]) || !EQ(pass, fields[3])))
 	    continue;
 
 	PERMcanread = strchr(fields[1], 'R') != NULL;
 	PERMcanpost = strchr(fields[1], 'P') != NULL;
-	(void)strcpy(PERMuser, fields[2]);
-	(void)strcpy(PERMpass, fields[3]);
+	(void)strcpy(PERMuser, user ? user : fields[2]);
+	(void)strcpy(PERMpass, pass ? pass : fields[3]);
 	(void)strcpy(accesslist, fields[4]);
 	found = TRUE;
     }
@@ -338,11 +393,26 @@ StartConnection(accesslist)
     if (getpeername(STDIN, (struct sockaddr *)&sin, &length) < 0) {
 	if (!isatty(STDIN)) {
 	    syslog(L_ERROR, "%s cant getpeername %m", "?");
+            (void)strcpy(ClientHost, "?"); /* so stats generation looks correct. */
 	    Printf("%d I can't get your name.  Goodbye.\r\n", NNTP_ACCESS_VAL);
 	    ExitWithStats(1);
 	}
 	(void)strcpy(ClientHost, "stdin");
     }
+#if defined(AF_DECnet)
+    else if (sin.sin_family == AF_DECnet) {
+	char *p, *MyDECnetDomain;
+	(void) strcpy(ClientHost, getenv("REMNODE"));
+	for (p = ClientHost; *p; p++)
+	    if (CTYPE(isupper, *p))
+		*p = tolower(*p);
+#define _CONF_DECNETDOMAIN "decnetdomain"
+	if ((MyDECnetDomain = GetConfigValue(_CONF_DECNETDOMAIN)) != NULL) {
+	    (void)strcat(ClientHost, ".");
+	    (void)strcat(ClientHost, MyDECnetDomain);
+	}
+    }
+#endif
     else {
 	if (sin.sin_family != AF_INET) {
 	    syslog(L_ERROR, "%s bad_address_family %ld",
@@ -352,32 +422,48 @@ StartConnection(accesslist)
 	}
 
 	/* Get client's name. */
-#if	!defined(DO_NNRP_GETHOSTBYADDR)
-	(void)strcpy(ClientHost, inet_ntoa(sin.sin_addr));
-#else
+#if	defined(DO_NNRP_GETHOSTBYADDR)
 	if (!Address2Name(&sin.sin_addr, ClientHost, sizeof ClientHost)) {
 	    (void)strcpy(ClientHost, inet_ntoa(sin.sin_addr));
-	    syslog(L_ERROR, "? cant gethostbyaddr %s %m", ClientHost);
+	    syslog(L_NOTICE,
+		"? cant gethostbyaddr %s %m -- using IP address for access",
+		ClientHost);
 	}
 	else {
 	    ClientAddr = buff;
 	    (void)strcpy(buff, inet_ntoa(sin.sin_addr));
 	}
-#endif /* !defined(DO_NNRP_GETHOSTBYADDR) */
+#else
+	(void)strcpy(ClientHost, inet_ntoa(sin.sin_addr));
+#endif /* defined(DO_NNRP_GETHOSTBYADDR) */
     }
 
+    strncpy (LogName,ClientHost,sizeof(LogName)-1) ;
+
     syslog(L_NOTICE, "%s connect", ClientHost);
     if (!PERMinfile(ClientHost, ClientAddr, (char *)NULL, (char *)NULL,
 	    accesslist)) {
 	syslog(L_NOTICE, "%s no_access", ClientHost);
 	Printf("%d You are not in my access file.  Goodbye.\r\n",
-	NNTP_ACCESS_VAL);
+	    NNTP_ACCESS_VAL);
 	ExitWithStats(1);
     }
 }
 
 
 #if	!defined(VAR_NONE)
+
+#if	defined(VAR_VARARGS)
+#if	defined(lint)
+#define START_VARARG(fmt, vp, type)	va_start(vp); fmt = NULL
+#else
+#define START_VARARG(fmt, vp, type)	va_start(vp); fmt = va_arg(vp, type)
+#endif	/* defined(lint) */
+#endif	/* defined(VAR_VARARGS) */
+#if	defined(VAR_STDARGS)
+#define START_VARARG(fmt, vp, type)	va_start(vp, fmt)
+#endif	/* defined(VAR_STDARGS) */
+
 /*
 **  Send a reply, possibly with debugging output.
 */
@@ -397,33 +483,26 @@ Reply(char *fmt, ...)
     char		buff[2048];
 #if	defined(VAR_VARARGS)
     register char	*fmt;
-
-    va_start(vp);
-#if	defined(lint)
-    fmt = NULL;
-#else
-    fmt = va_arg(vp, char *);
-#endif	/* defined(lint) */
 #endif	/* defined(VAR_VARARGS) */
-#if	defined(VAR_STDARGS)
-    va_start(vp, fmt);
-#endif	/* defined(VAR_STDARGS) */
+
+    START_VARARG(fmt, vp, char*);
     (void)vprintf(fmt, vp);
+    va_end(vp);
 
     if (Tracing) {
 	oerrno = errno;
+	START_VARARG(fmt, vp, char*);
 
 	/* Copy output, but strip trailing CR-LF. */
 	(void)vsprintf(buff, fmt, vp);
 	p = buff + strlen(buff) - 1;
 	while (p >= buff && (*p == '\n' || *p == '\r'))
 	    *p-- = '\0';
-
 	syslog(LOG_DEBUG, "%s > %s", ClientHost, buff);
+
+	va_end(vp);
 	errno = oerrno;
     }
-
-    va_end(vp);
 }
 #endif	/* !defined(VAR_NONE) */
 
@@ -453,9 +532,10 @@ Usage()
 
 /* ARGSUSED0 */
 int
-main(argc, argv)
+main(argc, argv, env)
     int			argc;
     char		*argv[];
+    char		*env[];
 {
 #if	NNRP_LOADLIMIT > 0
     int			load;
@@ -469,22 +549,27 @@ main(argc, argv)
     register int	i;
     char		*Reject;
     char		accesslist[BIG_BUFFER];
+    int			timeout;
 
-#if	!defined(hpux)
+#if	!defined(HPUX)
     /* Save start and extent of argv for TITLEset. */
     TITLEstart = argv[0];
     TITLEend = argv[argc - 1] + strlen(argv[argc - 1]) - 1;
-#endif	/* !defined(hpux) */
+#endif	/* !defined(HPUX) */
 
-    /* Parse arguments. */
+    /* Parse arguments.   Must COPY() optarg if used because the
+     * TITLEset() routine would clobber it! */
     Reject = NULL;
-    while ((i = getopt(argc, argv, "m:r:s:t")) != EOF)
+    while ((i = getopt(argc, argv, "S:r:s:t")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
 	    /* NOTREACHED */
-	case 'm':			/* Remote NNTP master */
-	    RemoteMaster = optarg;
+	case 'i':			/* Initial command */
+	    PushedBack = COPY(optarg);
+	    break;
+	case 'S':			/* We're a slave to NNTP master */
+	    RemoteMaster = COPY(optarg);
 	    break;
 	case 's':			/* Unused title string */
 	    break;
@@ -521,6 +606,8 @@ main(argc, argv)
     }
 #endif	/* NNRP_LOADLIMIT > 0 */
 
+    strcpy (LogName,"?") ;
+    
     /* Ignore SIGPIPE, since we'll see closed connections with read. */
     (void)signal(SIGPIPE, SIG_IGN);
 
@@ -537,7 +624,7 @@ main(argc, argv)
     }
 
     /* Proceed with initialization. */
-    PERMneedauth = PERMuser[0] != '\0' && PERMpass != '\0';
+    PERMneedauth = PERMuser[0] != '\0' && PERMpass[0] != '\0';
     PERMspecified = NGgetlist(&PERMlist, accesslist);
     TITLEset("connect");
 
@@ -548,6 +635,7 @@ main(argc, argv)
 	ExitWithStats(0);
     }
 
+    ARTreadschema();
     if (!GetGroupList()) {
 	/* This shouldn't really happen. */
 	syslog(L_NOTICE, "%s cant getgrouplist %m", ClientHost);
@@ -555,49 +643,66 @@ main(argc, argv)
 	ExitWithStats(1);
     }
 
+#if defined(DO_PERL)
+    /* Load the Perl code */
+    PERLsetup(NULL, _PATH_PERL_FILTER_NNRPD, "filter_post");
+    PerlFilter (TRUE) ;
+#endif /* defined(DO_PERL) */
+
     Reply("%d %s InterNetNews NNRP server %s ready (%s).\r\n",
 	   PERMcanpost ? NNTP_POSTOK_VAL : NNTP_NOPOSTOK_VAL,
 	   MyHostName, INNVersion(),
 	   PERMcanpost ? "posting ok" : "no posting");
 
     /* Main dispatch loop. */
-    for (av = NULL; ; ) {
+    for (timeout = INITIAL_TIMEOUT, av = NULL; ; timeout = CLIENT_TIMEOUT) {
 	(void)fflush(stdout);
 	if (ChangeTrace) {
 	    Tracing = Tracing ? FALSE : TRUE;
-	    syslog(L_TRACE, "trace %sabled", Tracing ? "en" : "dis");
+	    syslog(L_TRACE, "trace %abled", Tracing ? "en" : "dis");
 	    ChangeTrace = FALSE;
 	}
-	switch (r = READline(buff, (int)sizeof buff, CLIENT_TIMEOUT)) {
-	default:
-	    syslog(L_ERROR, "%s internal %d in main", ClientHost, r);
-	    /* FALLTHROUGH */
-	case RTtimeout:
-	    syslog(L_NOTICE, "%s timeout", ClientHost);
-	    Printf("%d Timeout after %d seconds, closing connection.\r\n",
-		   NNTP_TEMPERR_VAL, CLIENT_TIMEOUT);
-	    ExitWithStats(1);
-	    break;
-	case RTlong:
-	    Reply("%d Line too long\r\n", NNTP_BAD_COMMAND_VAL);
-	    continue;
-	case RTok:
-	    /* Do some input processing, check for blank line. */
-	    if (buff[0] == '\0')
+	if (PushedBack) {
+	    if (PushedBack[0] == '\0')
 		continue;
 	    if (Tracing)
-		syslog(L_TRACE, "%s < %s", ClientHost, buff);
-	    ac = Argify(buff, &av);
-	    if (ac == 0)
-		continue;
-	    break;
-	case RTeof:
-	    /* Handled below. */
-	    break;
+		syslog(L_TRACE, "%s < %s", ClientHost, PushedBack);
+	    ac = Argify(PushedBack, &av);
 	}
+	else
+	    switch (r = READline(buff, (int)sizeof buff, timeout)) {
+	    default:
+		syslog(L_ERROR, "%s internal %d in main", ClientHost, r);
+		/* FALLTHROUGH */
+	    case RTtimeout:
+		if (timeout < CLIENT_TIMEOUT)
+		    syslog(L_NOTICE, "%s timeout short", ClientHost);
+		else
+		    syslog(L_NOTICE, "%s timeout", ClientHost);
+		Printf("%d Timeout after %d seconds, closing connection.\r\n",
+		       NNTP_TEMPERR_VAL, timeout);
+		ExitWithStats(1);
+		break;
+	    case RTlong:
+		Reply("%d Line too long\r\n", NNTP_BAD_COMMAND_VAL);
+		continue;
+	    case RTok:
+		/* Do some input processing, check for blank line. */
+		if (Tracing)
+		    syslog(L_TRACE, "%s < %s", ClientHost, buff);
+		if (buff[0] == '\0')
+		    continue;
+		ac = Argify(buff, &av);
+		break;
+	    case RTeof:
+		/* Handled below. */
+		break;
+	    }
 
 	/* Client gone? */
-	if (r == RTeof || caseEQ(av[0], "quit"))
+	if (r == RTeof)
+	    break;
+	if (ac == 0 || caseEQ(av[0], "quit"))
 	    break;
 
 	/* Find command. */
@@ -617,7 +722,7 @@ main(argc, argv)
 	if ((cp->Minac != CMDany && ac < cp->Minac)
 	 || (cp->Maxac != CMDany && ac > cp->Maxac)) {
 	    Reply("%d %s\r\n",
-		NNTP_BAD_COMMAND_VAL,  cp->Help ? cp->Help : "Usage error");
+		NNTP_SYNTAX_VAL,  cp->Help ? cp->Help : "Usage error");
 	    continue;
 	}
 
