@@ -1,31 +1,22 @@
-/*  $Revision: 1.15 $
+/*  $Revision: 1.30 $
 **
 **  Check article, send it to the local server.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "nnrpd.h"
 #if	defined(DO_NEED_TIME)
 #include <time.h>
 #endif	/* defined(DO_NEED_TIME) */
 #include <sys/time.h>
 
+#include "post.h"
+
 #define FLUSH_ERROR(F)		(fflush((F)) == EOF || ferror((F)))
 #define HEADER_DELTA		20
 
-typedef enum _HEADERTYPE {
-    HTobs,
-    HTreq,
-    HTstd
-} HEADERTYPE;
-
-typedef struct _HEADER {
-    STRING	Name;
-    BOOL	CanSet;
-    HEADERTYPE	Type;
-    int		Size;
-    char	*Value;
-} HEADER;
-
-
 STATIC char	Error[SMBUF];
 STATIC char	NGSEPS[] = NG_SEPARATOR;
 STATIC char	**OtherHeaders;
@@ -36,7 +27,7 @@ STATIC STRING	BadDistribs[] = {
     BAD_DISTRIBS
 };
 
-STATIC HEADER	Table[] = {
+HEADER	Table[] = {
     /* 	Name			Canset	Type	*/
     {	"Path",			TRUE,	HTstd },
 #define _path		 0
@@ -74,6 +65,12 @@ STATIC HEADER	Table[] = {
 #define _replyto	16
     {	"NNTP-Posting-Host",	FALSE,	HTstd },
 #define _nntpposthost	17
+    {	"Mime-Version",		TRUE,	HTstd },
+#define _mimeversion	18
+    {	"Content-Type",		TRUE,	HTstd },
+#define _contenttype	19
+    {	"Content-Transfer-Encoding", TRUE, HTstd },
+#define _contenttransferencoding 20
     {	"Xref",			FALSE,	HTstd },
     {	"Summary",		TRUE,	HTstd },
     {	"Keywords",		TRUE,	HTstd },
@@ -84,7 +81,7 @@ STATIC HEADER	Table[] = {
     {	"Relay-Version",	FALSE,	HTobs },
 };
 
-#define HDR(_x)	(Table[(_x)].Value)
+HEADER *EndOfTable = ENDOF(Table);
 
 
 
@@ -204,10 +201,6 @@ CheckControl(ctrl)
     save = *p;
     *p = '\0';
 
-    if (EQ(ctrl, "ihave") || EQ(ctrl, "sendme")) {
-	(void)sprintf(Error, "Unsupported \"%s\" control message", ctrl);
-	return Error;
-    }
     if (EQ(ctrl, "cancel")) {
 	for (q = p + 1; ISWHITE(*q); q++)
 	    continue;
@@ -215,7 +208,8 @@ CheckControl(ctrl)
 	    return "Message-ID missing in cancel";
     }
     else if (EQ(ctrl, "sendsys") || EQ(ctrl, "senduuname")
-	  || EQ(ctrl, "version")
+	  || EQ(ctrl, "version") || EQ(ctrl, "checkgroups")
+	  || EQ(ctrl, "ihave") || EQ(ctrl, "sendme")
 	  || EQ(ctrl, "newgroup") || EQ(ctrl, "rmgroup"))
 	/* SUPPRESS 530 *//* Empty body for statement */
 	;
@@ -261,11 +255,13 @@ ProcessHeaders(linecount)
     int			linecount;
 {
     static char		MONTHS[] = "JanFebMarAprMayJunJulAugSepOctNovDec";
-    static char		subjbuff[NNTP_STRLEN];
     static char		datebuff[40];
     static char		orgbuff[SMBUF];
     static char		linebuff[40];
-    static char		pathbuff[SMBUF];
+    static char		sendbuff[SMBUF];
+    static char		mimeversion[SMBUF];
+    static char		mimetype[SMBUF];
+    static char		mimeencoding[SMBUF];
     register HEADER	*hp;
     register char	*p;
     time_t		t;
@@ -287,8 +283,15 @@ ProcessHeaders(linecount)
     }
 
 #if	defined(DO_NNRP_AUTH_SENDER)
-    /* Zap the Sender? */
-    if (!PERMauthorized)
+    /* If authorized and we didn't get a sender, add the header based on
+     * our info.  If not authorized, zap the Sender so we don't put out
+     * unauthenticated data. */
+    if (PERMauthorized && HDR(_sender) == NULL) {
+	(void)sprintf(sendbuff, "%s@%s",
+	    PERMuser ? PERMuser : "UNKNOWN", ClientHost);
+	HDR(_sender) = sendbuff;
+    }
+    else if (!PERMauthorized)
 	HDR(_sender) = NULL;
 #endif	/* defined(DO_NNRP_AUTH_SENDER) */
 
@@ -301,7 +304,7 @@ ProcessHeaders(linecount)
     if (HDR(_date) == NULL) {
 	if ((gmt = gmtime(&Now.time)) == NULL)
 	    return "Can't get the time";
-	(void)sprintf(datebuff, "%d %3.3s %d %02.2d:%02.2d:%02.2d GMT",
+	(void)sprintf(datebuff, "%d %3.3s %d %02d:%02d:%02d GMT",
 	    gmt->tm_mday, &MONTHS[3 * gmt->tm_mon], 1900 + gmt->tm_year,
 	    gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
 	HDR(_date) = datebuff;
@@ -315,22 +318,19 @@ ProcessHeaders(linecount)
 
     /* Newsgroups are checked later. */
 
-    /* Set Subject; Control overrides the subject. */
     if (HDR(_control)) {
 	if ((error = CheckControl(HDR(_control))) != NULL)
 	    return error;
-	(void)sprintf(subjbuff, "cmsg %s", HDR(_control));
-	HDR(_subject) = subjbuff;
     }
     else {
 	p = HDR(_subject);
 	if (p == NULL)
 	    return "Required \"Subject\" header is missing";
-	if (EQn(p, "cmsg ", 5)) {
-	    HDR(_control) = p + 5;
-	    if ((error = CheckControl(HDR(_control))) != NULL)
-		return error;
-	}
+        if (EQn(p, "cmsg ", 5)) {
+            HDR(_control) = p + 5;
+            if ((error = CheckControl(HDR(_control))) != NULL)
+                return error;
+        }
 #if	0
 	if (EQn(p, "Re: ", 4) && HDR(_references) == NULL)
 	    return "Article starts with \"Re: \" but has no references";
@@ -349,15 +349,23 @@ ProcessHeaders(linecount)
 
     /* Set Path */
     if (HDR(_path) == NULL) {
-	if ((p = GetConfigValue(_CONF_PATHHOST)) == NULL)
-	    return "Can't get host name";
-	if (*p)
-	    (void)sprintf(pathbuff, "%s!%s", p, NEWSMASTER);
-	else
-	    (void)strcpy(pathbuff, NEWSMASTER);
-	HDR(_path) = pathbuff;
-	
+	/* Note that innd will put host name here for us. */
+	HDR(_path) = PATHMASTER;
+    } else {
+#if 0
+        /* Here's where to do Path changes for new Posts. */
+        char *newpath;
+        if ((newpath = strrchr(HDR(_path), '!')) != NULL) {
+            newpath++;
+            if (*newpath == '\0') {
+                HDR(_path) = NEWSMASTER;
+            } else {
+                HDR(_path) = newpath;
+            }
+        }
+#endif
     }
+    
 
     /* Reply-To; left alone. */
     /* Sender; set above. */
@@ -389,6 +397,29 @@ ProcessHeaders(linecount)
     /* Summary; left alone. */
     /* Approved; left alone. */
 
+    /* MIME headers. */
+    if (HDR(_mimeversion) == NULL
+     && (p = GetConfigValue(_CONF_MIMEVERSION)) != NULL) {
+	(void)strcpy(mimeversion, p);
+	HDR(_mimeversion) = mimeversion;
+
+	/* Set Content-Type */
+	if (HDR(_contenttype) == NULL) {
+	    if ((p = GetConfigValue(_CONF_CONTENTTYPE)) == NULL)
+		return "Can't get \"Content-Type\" header";
+	    (void)strcpy(mimetype, p);
+	    HDR(_contenttype) = mimetype;
+	}
+
+	/* Set Content-Transfer-Encoding */
+	if (HDR(_contenttransferencoding) == NULL) {
+	    if ((p = GetConfigValue(_CONF_ENCODING)) == NULL)
+		return "Can't get \"Content-Transfer-Encoding\" header";
+	    (void)strcpy(mimeencoding, p);
+	    HDR(_contenttransferencoding) = mimeencoding;
+	}
+    }
+
     /* Set Lines */
     (void)sprintf(linebuff, "%d", linecount);
     HDR(_lines) = linebuff;
@@ -429,6 +460,12 @@ CheckIncludedText(p, lines)
 	case '>':
 	    i++;
 	    break;
+	case '|':
+	    i++;
+	    break;
+	case ':':
+	    i++;
+	    break;
 	case '<':
 	    i--;
 	    break;
@@ -436,7 +473,7 @@ CheckIncludedText(p, lines)
 	if ((p = strchr(p + 1, '\n')) == NULL)
 	    break;
     }
-    if (i * 2 > lines)
+    if ((i * 2 > lines) && (lines > 40))
 	return "Article not posted -- more included text than new text";
     return NULL;
 }
@@ -460,7 +497,7 @@ MailArticle(group, article)
     char		buff[SMBUF];
 
     /* Try to get the address first. */
-    if ((address = GetModeratorAddress(group)) == NULL) {
+    if ((address = GetModeratorAddress(NULL, NULL, group)) == NULL) {
 	(void)sprintf(Error, "No mailing address for \"%s\" -- %s",
 		group, "ask your news administrator to fix this");
 	return Error;
@@ -527,7 +564,9 @@ ValidNewsgroups(hdr, article)
     register GROUPENTRY	*gp;
     register BOOL	approved;
     struct _DDHANDLE	*h;
+    char      *grplist[2];
     BOOL		IsNewgroup;
+    BOOL		FoundOne;
 
     p = HDR(_control);
     IsNewgroup = p && EQn(p, "newgroup", 8);
@@ -539,26 +578,23 @@ ValidNewsgroups(hdr, article)
     approved = HDR(_approved) != NULL || article == NULL;
 
     Error[0] = '\0';
+    FoundOne = FALSE;
     h = DDstart((FILE *)NULL, (FILE *)NULL);
     do {
 #if	defined(DO_MERGE_TO_GROUPS)
 	if (p[0] == 't' && p[1] == 'o' && p[2] == '.')
 	    p = "to";
 #endif	/* defined(DO_MERGE_TO_GROUPS) */
-	if ((gp = GRPfind(p)) == NULL) {
-	    if (IsNewgroup)
-		continue;
-	    DISPOSE(groups);
-	    (void)sprintf(Error, "No such newsgroup as \"%s\"", p);
-	    DISPOSE(DDend(h));
-	    return Error;
-	}
+	if ((gp = GRPfind(p)) == NULL)
+	    continue;
+	FoundOne = TRUE;
 	DDcheck(h, p);
 	switch (gp->Flag) {
 	case NF_FLAG_OK:
 	    break;
 	case NF_FLAG_MODERATED:
 	    if (!approved) {
+		DISPOSE(groups);
 		DISPOSE(DDend(h));
 		return MailArticle(gp->Name, article);
 	    }
@@ -574,12 +610,21 @@ ValidNewsgroups(hdr, article)
 	case NF_FLAG_ALIAS:
 	    (void)sprintf(Error,
 		    "The newsgroup \"%s\" has been renamed to \"%s\".\n",
-		    gp, gp->Alias);
+		    p, gp->Alias);
 	    break;
 	}
+        if (PERMspecified) {
+	    grplist[0] = p;
+	    grplist[1] = NULL;
+	    if (!PERMmatch(PERMdefault, PERMlist, grplist)) {
+		sprintf(Error,"%s: %s\r\n", NNTP_NOSUCHGROUP,p);
+	    }
+        }
     } while ((p = strtok((char *)NULL, NGSEPS)) != NULL);
     DISPOSE(groups);
 
+    if (!FoundOne && !IsNewgroup)
+	(void)sprintf(Error, "No valid newsgroups in \"%s\"", hdr);
     if (Error[0]) {
 	DISPOSE(DDend(h));
 	return Error;
@@ -613,6 +658,98 @@ SendQuit(FromServer, ToServer)
 }
 
 
+/*
+**  Offer the article to the server, return its reply.
+*/
+STATIC int
+OfferArticle(buff, buffsize, FromServer, ToServer)
+    char		*buff;
+    int			buffsize;
+    FILE		*FromServer;
+    FILE		*ToServer;
+{
+    static char		CANTSEND[] = "Can't send %s to server, %s";
+
+    (void)fprintf(ToServer, "ihave %s\r\n", HDR(_messageid));
+    if (FLUSH_ERROR(ToServer)
+     || fgets(buff, buffsize, FromServer) == NULL) {
+	(void)sprintf(buff, CANTSEND, "IHAVE", strerror(errno));
+	return -1;
+    }
+    return atoi(buff);
+}
+
+
+/*
+**  Spool article to temp file.
+*/
+STATIC STRING
+Spoolit(article, Error)
+    char 		*article;
+    char		*Error;
+{
+    static char		CANTSPOOL[NNTP_STRLEN+2];
+    static char		SPOOLNEWS[] = _PATH_SPOOLNEWS;
+    register HEADER	*hp;
+    register FILE	*F = NULL;
+    register int	i;
+    char		temp[BUFSIZ];
+    struct stat		Sb;
+
+    /* Initialize the returned error message */
+    sprintf(CANTSPOOL, "%s and can't write text to local spool file", Error);
+
+    /* Try to write it to the spool dir. */
+    if (stat(SPOOLNEWS, &Sb) >= 0 && S_ISDIR(Sb.st_mode)) {
+	TempName(SPOOLNEWS, temp);
+	(void)umask(0);
+	if ((i = open(temp, O_WRONLY | O_CREAT, BATCHFILE_MODE)) >0 ) {
+            if ((F = fdopen(i, "w")) == NULL) {
+                close(i);
+            }
+        }
+        if (F==NULL) {
+            sprintf(CANTSPOOL, "%s and can't create local spool file", Error);
+            return CANTSPOOL;
+        }
+    }
+
+    /* Write the headers and a blank line. */
+    for (hp = Table; hp < ENDOF(Table); hp++)
+	if (hp->Value) {
+	    (void)fprintf(F, "%s: %s\n", hp->Name, hp->Value);
+	    if (FLUSH_ERROR(F)) {
+		(void)pclose(F);
+		return CANTSPOOL;
+	    }
+	}
+    for (i = 0; i < OtherCount; i++) {
+	(void)fprintf(F, "%s\n", OtherHeaders[i]);
+	if (FLUSH_ERROR(F)) {
+	    (void)pclose(F);
+	    return CANTSPOOL;
+	}
+    }
+    (void)fprintf(F, "\n");
+
+    /* Write the article body */
+    i = strlen(article);
+    if (fwrite((POINTER)article, (SIZE_T)1, (SIZE_T)i, F) != i) {
+        fclose(F);
+        return CANTSPOOL;
+    }
+
+    /* Flush and catch any errors */
+    if (FLUSH_ERROR(F)) {
+	(void)fclose(F);
+	return CANTSPOOL;
+    }
+
+    /* Article has been spooled */
+    fclose(F);
+    return NULL;
+}
+
 STRING
 ARTpost(article, idbuff)
     char		*article;
@@ -625,7 +762,7 @@ ARTpost(article, idbuff)
     register HEADER	*hp;
     FILE		*ToServer;
     FILE		*FromServer;
-    char		buff[NNTP_STRLEN + 2];
+    char		buff[NNTP_STRLEN + 2], frombuf[SMBUF];
     STRING		error;
 
     /* Set up the other headers list. */
@@ -657,6 +794,17 @@ ARTpost(article, idbuff)
     if ((error = ValidNewsgroups(HDR(_newsgroups), article)) != NULL
      || WasMailed)
 	return error;
+    strcpy(frombuf, HDR(_from));
+    HeaderCleanFrom(frombuf);
+    p = strchr(frombuf, '@');
+    if (p) {
+	strcpy(frombuf, p+1);
+	p = strrchr(frombuf, '.');
+	if (!p)
+	    return "From: address not in Internet syntax";
+    }
+    else
+	return "From: address not in Internet syntax";
     if ((p = HDR(_followupto)) != NULL
      && !EQ(p, "poster")
      && (error = ValidNewsgroups(p, (char *)NULL)) != NULL)
@@ -670,37 +818,58 @@ ARTpost(article, idbuff)
     }
 #endif	/* LOCAL_MAX_ARTSIZE > 0 */
 
+#if defined(DO_PERL)
+    /* Calls the Perl subroutine for headers management */
+    if ((p = (char *)HandleHeaders()) != NULL)
+        return p;
+#endif /* defined(DO_PERL) */
+
     /* Open a local connection to the server. */
-    i = RemoteMaster
-	? NNTPconnect(RemoteMaster, &FromServer, &ToServer, buff)
-	: NNTPlocalopen(&FromServer, &ToServer, buff);
+    if (RemoteMaster)
+	i = NNTPconnect(RemoteMaster, &FromServer, &ToServer, buff);
+    else {
+#if	defined(DO_HAVE_UNIX_DOMAIN)
+	i = NNTPlocalopen(&FromServer, &ToServer, buff);
+#else
+	i = NNTPremoteopen(&FromServer, &ToServer, buff);
+#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+    }
+
+    /* If we cannot open the connection, initialize the error message and
+     * attempt to recover from this by spooling it locally */
     if (i < 0) {
 	if (buff[0])
 	    (void)strcpy(Error, buff);
 	else
 	    (void)sprintf(Error, CANTSEND, "connect request", strerror(errno));
-	return Error;
+        return Spoolit(article,Error);
     }
+    if (Tracing)
+	syslog(L_TRACE, "%s post_connect %s",
+	    ClientHost, RemoteMaster ? RemoteMaster : "localhost");
 
     /* The code below has too many (void) casts for my tastes.  At least
      * they are all inside cases that are most likely never going to
      * happen -- for example, if the server crashes. */
 
     /* Offer article to server. */
-    (void)fprintf(ToServer, "ihave %s\r\n", HDR(_messageid));
-    if (FLUSH_ERROR(ToServer)
-     || fgets(buff, sizeof buff, FromServer) == NULL) {
-	(void)sprintf(Error, CANTSEND, "IHAVE", strerror(errno));
-	(void)fclose(FromServer);
-	(void)fclose(ToServer);
-	return Error;
+    i = OfferArticle(buff, sizeof buff, FromServer, ToServer);
+    if (i == NNTP_AUTH_NEEDED_VAL) {
+        /* Send authorization. */
+        if (NNTPsendpassword(RemoteMaster, FromServer, ToServer) < 0) {
+            (void)sprintf(Error, "Can't authorize with %s",
+                          RemoteMaster ? RemoteMaster : "innd");
+            return Spoolit(article,Error);
+        }
+        i = OfferArticle(buff, sizeof buff, FromServer, ToServer);
     }
-
-    if (atoi(buff) != NNTP_SENDIT_VAL) {
-	(void)strcpy(Error, buff);
-	SendQuit(FromServer, ToServer);
-	return Error;
+    if (i != NNTP_SENDIT_VAL) {
+        (void)strcpy(Error, buff);
+        SendQuit(FromServer, ToServer);
+        return (i != NNTP_HAVEIT_VAL ? Spoolit(article, Error) : Error) ;
     }
+    if (Tracing)
+	syslog(L_TRACE, "%s post starting", ClientHost);
 
     /* Write the headers and a blank line. */
     for (hp = Table; hp < ENDOF(Table); hp++)
@@ -713,7 +882,7 @@ ARTpost(article, idbuff)
 	(void)sprintf(Error, CANTSEND, "headers", strerror(errno));
 	(void)fclose(FromServer);
 	(void)fclose(ToServer);
-	return Error;
+	return Spoolit(article, Error);
     }
 
     /* Send the article, get the server's reply. */
@@ -722,20 +891,18 @@ ARTpost(article, idbuff)
 	(void)sprintf(Error, CANTSEND, "article", strerror(errno));
 	(void)fclose(FromServer);
 	(void)fclose(ToServer);
-	return Error;
+	return Spoolit(article, Error);
     }
 
     /* Did the server want the article? */
-    if (atoi(buff) != NNTP_TOOKIT_VAL) {
+    if ((i = atoi(buff)) != NNTP_TOOKIT_VAL) {
 	(void)strcpy(Error, buff);
 	SendQuit(FromServer, ToServer);
-	return Error;
+	return (i != NNTP_REJECTIT_VAL ? Spoolit(article, Error) : Error) ;
     }
 
     /* Send a quit and close down */
     SendQuit(FromServer, ToServer);
-    (void)fclose(FromServer);
-    (void)fclose(ToServer);
     if (idbuff)
 	(void)strcpy(idbuff, HDR(_messageid));
     return NULL;
