@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# $Id: innlog.pl,v 1.39 1996/12/17 13:17:35 brister Exp $
+# $Id: innlog.pl,v 1.38 1996/11/28 12:49:37 brister Exp $
 #
 # This version by Greg Patten <greg@loose.apana.org.au>
 #
@@ -390,8 +390,8 @@ while ($line = <>)
 	}
 
 	if ($Fld[6] eq "rejecting[perl]") {
-	    @line = splice(@Fld, 8); 
-	    shift(@line) while ($#line >= $[ && $line[$[] ne '437');
+	    @line = splice(@Fld, 8);
+	    shift(@line) while ($#line >= 0 && $line[0] ne '437');
 	    shift(@line);
 	    if ($#line >= 0) {
 	    	$innd_perl_reject{join(' ', @line)}++;
